#%%
from pathlib import Path
from src.read_swc import read_swc_offset
import numpy as np
from itertools import islice
import pickle
import random

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

#%%
def calc_distance(frag_1, frag_2):
    """Calculate the distance between the closest two nodes in two fragments (single linkage)
    
    Arguments:
        frag_1 {pandas dataframe} -- node coordinates from read_swc
        frag_2 {pandas dataframe} -- node coordinates from read_swc
    """
    coords_1 = frag_1.as_matrix(columns=['x','y','z'])
    coords_2 = frag_2.as_matrix(columns=['x','y','z'])
    min_dist = np.inf
    pair = [-1,-1]

    for i in range(coords_1.shape[0]):
        coord_1 = coords_1[i,:]
        for j in range(coords_2.shape[0]):
            coord_2 = coords_2[j,:]
            dist = np.linalg.norm(coord_1-coord_2)
            if dist < min_dist:
                min_dist = dist
                pair = [i,j]
    return min_dist, pair

#%%
#change this
path = Path('C:/Users/Thomas Athey/Documents/Labs/Labs/Miller/mouselight/data/2018-08-01/prob0_swcs2/frags')

files = list(path.glob('**/*.swc'))
n_frags = len(files)
distances = np.zeros(n_frags)
i_prime = random.randint(10,n_frags)

for i,file_source in enumerate(files):
    if i != i_prime:
        continue
    df_src, _, cc_src, branch_src = read_swc_offset(file_source)
    for j,file_target in enumerate(files): 
        df_target, _, _, _ = read_swc_offset(file_target)
        min_dist, pair = calc_distance(df_src,df_target)
        distances[j] = min_dist
        if j%2500==0:
            print(str(i) + '--' + str(j))
    file_name = str(cc_src) + '_' + str(branch_src) + '_' + 'distances.pickle'
    with open(file_name, 'wb') as f:
        pickle.dump([distances], f)
#%%

dist_path = Path('C:/Users/Thomas Athey/Documents/Labs/Labs/Miller/mouselight/statistics/distances')
dist_files = list(dist_path.glob('**/*.pickle'))
distances = np.zeros((len(dist_files),n_frags))
for i,dist_file in enumerate(dist_files):
    distances[i,:] = pickle.load(open(dist_file,'rb'))[0]

#%%
sorted_dist = np.sort(distances,axis=1)[:,1:]
ns = np.arange(100)
f = plt.figure(figsize=(6,6))
for i in range(sorted_dist.shape[0]):
    plt.plot(ns,sorted_dist[i,ns])
plt.xlabel('Proximity Rank')
plt.ylabel('Distance')
plt.show()

#%%
mins = np.amin(sorted_dist,axis=1)
maxs = np.amax(sorted_dist,axis=1)
print(np.mean(mins))
print(np.mean(maxs))
print(np.mean(sorted_dist))

#%%

f = plt.figure(figsize=(6,6))
plt.hist(sorted_dist.flatten(),bins=1000)
plt.xlabel('Distance')
plt.ylabel('Count')

#%%
